package com.yossi.kosherappstore

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.net.Uri
import android.os.Bundle
import android.provider.OpenableColumns
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.widget.Button
import android.widget.EditText
import android.widget.RadioButton
import android.widget.RadioGroup
import android.widget.TextView
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.edit
import com.google.android.material.switchmaterial.SwitchMaterial
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import java.io.BufferedInputStream
import java.io.BufferedOutputStream
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.io.IOException
import java.io.InputStream
import java.io.OutputStream
import java.lang.reflect.Type
import java.util.UUID
import java.util.zip.ZipEntry
import java.util.zip.ZipInputStream
import java.util.zip.ZipOutputStream

// Importים לקבועים מ-MainActivity
import com.yossi.kosherappstore.MainActivity.Companion.PREFS_NAME
import com.yossi.kosherappstore.MainActivity.Companion.KEY_APPS_LIST
import com.yossi.kosherappstore.MainActivity.Companion.KEY_SETTINGS_PASSWORD
import com.yossi.kosherappstore.MainActivity.Companion.DEFAULT_PASSWORD
import com.yossi.kosherappstore.MainActivity.Companion.KEY_INSTALL_LOCATION
import com.yossi.kosherappstore.MainActivity.Companion.KEY_USE_ROOT_INSTALL
import com.yossi.kosherappstore.MainActivity.Companion.KEY_NORMAL_INSTALL_MODE
import com.yossi.kosherappstore.MainActivity.Companion.KEY_ROOT_INSTALL_METHOD
import com.yossi.kosherappstore.MainActivity.Companion.ROOT_METHOD_PM_INSTALL
import com.yossi.kosherappstore.MainActivity.Companion.ROOT_METHOD_COPY_TO_SYSTEM
import com.yossi.kosherappstore.MainActivity.Companion.KEY_ENABLE_SEND_FEEDBACK
import com.yossi.kosherappstore.MainActivity.Companion.KEY_ABOUT_TEXT
import com.yossi.kosherappstore.MainActivity.Companion.DEFAULT_ABOUT_TEXT


class SettingsActivity : AppCompatActivity() {

    companion object {
        private const val TAG = "SettingsActivity"
        private const val MANIFEST_FILE_NAME = "store_manifest.json"
        private const val APKS_DIR_NAME = "apks"
        private const val IMAGES_DIR_NAME = "images"
    }

    // Dialog related views
    private var editTextDialogAppName: EditText? = null
    private var editTextDialogAppDescription: EditText? = null
    private var editTextDialogAppPackageName: EditText? = null

    // APK/Image selection related
    private var currentApkUri: Uri? = null
    private var currentImageUri: Uri? = null

    // SharedPreferences and data
    private lateinit var sharedPreferences: SharedPreferences
    private val gson = Gson()

    // UI Elements from activity_settings.xml
    private lateinit var buttonAddApp: Button
    private lateinit var buttonManageStoreApps: Button
    private lateinit var switchNormalInstall: SwitchMaterial
    private lateinit var textViewNormalInstallLabel: TextView
    private lateinit var textViewRootSettingsLabel: TextView
    private lateinit var switchUseRootInstall: SwitchMaterial
    private lateinit var textViewRootInstallMethodLabel: TextView
    private lateinit var radioGroupRootInstallMethod: RadioGroup
    private lateinit var radioPmInstall: RadioButton
    private lateinit var radioCopyToSystem: RadioButton
    private lateinit var textViewCopyToSystemLocationLabel: TextView
    private lateinit var radioGroupInstallLocation: RadioGroup
    private lateinit var radioSystemApp: RadioButton
    private lateinit var radioSystemPrivApp: RadioButton
    private lateinit var switchEnableSendFeedback: SwitchMaterial
    private lateinit var editTextAboutText: EditText
    private lateinit var editTextOldPassword: EditText
    private lateinit var editTextNewPassword: EditText
    private lateinit var editTextConfirmNewPassword: EditText
    private lateinit var buttonChangePassword: Button
    private lateinit var buttonSaveSettingsAndExit: Button
    private lateinit var buttonExportStore: Button
    private lateinit var buttonImportStore: Button

    // ActivityResultLaunchers
    private lateinit var pickApkLauncher: ActivityResultLauncher<Array<String>>
    private lateinit var pickImageLauncher: ActivityResultLauncher<Array<String>>

    private val manageAppsResultLauncher =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == Activity.RESULT_OK) {
                setResult(Activity.RESULT_OK) // מודיע ל-MainActivity שהיו שינויים
            }
        }

    private val createDocumentLauncher =
        registerForActivityResult(ActivityResultContracts.CreateDocument("application/zip")) { uri: Uri? ->
            uri?.let { exportUri -> performExportStore(exportUri) }
        }

    private val openDocumentLauncher =
        registerForActivityResult(ActivityResultContracts.OpenDocument()) { uri: Uri? ->
            uri?.let { importUri ->
                AlertDialog.Builder(this)
                    .setTitle("ייבוא הגדרות")
                    .setMessage("פעולה זו תחליף את כל ההגדרות והאפליקציות הנוכחיות בחנות. האם אתה בטוח שברצונך להמשיך?")
                    .setPositiveButton("כן, ייבא") { _, _ -> performImportStore(importUri) }
                    .setNegativeButton("ביטול", null)
                    .show()
            }
        }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_settings)
        // אין title אם משתמשים ב-NoActionBar Theme שהוגדר במניפסט

        sharedPreferences = getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE)

        buttonAddApp = findViewById(R.id.buttonAddApp)
        buttonManageStoreApps = findViewById(R.id.buttonManageStoreApps)
        switchNormalInstall = findViewById(R.id.switchNormalInstall)
        textViewNormalInstallLabel = findViewById(R.id.textViewNormalInstallLabel)
        textViewRootSettingsLabel = findViewById(R.id.textViewRootSettingsLabel)
        switchUseRootInstall = findViewById(R.id.switchUseRootInstall)
        textViewRootInstallMethodLabel = findViewById(R.id.textViewRootInstallMethodLabel)
        radioGroupRootInstallMethod = findViewById(R.id.radioGroupRootInstallMethod)
        radioPmInstall = findViewById(R.id.radioPmInstall)
        radioCopyToSystem = findViewById(R.id.radioCopyToSystem)
        textViewCopyToSystemLocationLabel = findViewById(R.id.textViewCopyToSystemLocationLabel)
        radioGroupInstallLocation = findViewById(R.id.radioGroupInstallLocation)
        radioSystemApp = findViewById(R.id.radioSystemApp)
        radioSystemPrivApp = findViewById(R.id.radioSystemPrivApp)
        switchEnableSendFeedback = findViewById(R.id.switchEnableSendFeedback)
        editTextAboutText = findViewById(R.id.editTextAboutText)
        editTextOldPassword = findViewById(R.id.editTextOldPassword)
        editTextNewPassword = findViewById(R.id.editTextNewPassword)
        editTextConfirmNewPassword = findViewById(R.id.editTextConfirmNewPassword)
        buttonChangePassword = findViewById(R.id.buttonChangePassword)
        buttonSaveSettingsAndExit = findViewById(R.id.buttonSaveSettingsAndExit)
        buttonExportStore = findViewById(R.id.buttonExportStore)
        buttonImportStore = findViewById(R.id.buttonImportStore)

        setupFilePickers()
        setupListeners()
        loadPreferences()
        updateUiStates()
    }

    private fun setupFilePickers() {
        pickApkLauncher = registerForActivityResult(ActivityResultContracts.OpenDocument()) { uri: Uri? ->
            uri?.let {
                currentApkUri = it
                Toast.makeText(this, "APK נבחר: ${getFileNameFromUri(it)}", Toast.LENGTH_SHORT).show()
            }
        }
        pickImageLauncher = registerForActivityResult(ActivityResultContracts.OpenDocument()) { uri: Uri? ->
            uri?.let {
                currentImageUri = it
                Toast.makeText(this, "תמונה נבחרה: ${getFileNameFromUri(it)}", Toast.LENGTH_SHORT).show()
            }
        }
    }

    private fun setupListeners() {
        buttonAddApp.setOnClickListener {
            currentApkUri = null
            currentImageUri = null
            showAddAppDialog()
        }
        buttonManageStoreApps.setOnClickListener {
            val intent = Intent(this, ManageStoreAppsActivity::class.java)
            manageAppsResultLauncher.launch(intent)
        }
        switchNormalInstall.setOnCheckedChangeListener { _, isChecked ->
            if (isChecked && switchUseRootInstall.isChecked) switchUseRootInstall.isChecked = false
            updateUiStates()
        }
        switchUseRootInstall.setOnCheckedChangeListener { _, isChecked ->
            if (isChecked && switchNormalInstall.isChecked) switchNormalInstall.isChecked = false
            updateUiStates()
        }
        radioGroupRootInstallMethod.setOnCheckedChangeListener { _, _ -> updateUiStates() }
        buttonChangePassword.setOnClickListener { changePassword() }
        buttonSaveSettingsAndExit.setOnClickListener {
            savePreferences()
            setResult(Activity.RESULT_OK)
            finish()
        }
        buttonExportStore.setOnClickListener {
            val timestamp = System.currentTimeMillis()
            createDocumentLauncher.launch("KosherAppStore_Backup_$timestamp.zip")
        }
        buttonImportStore.setOnClickListener {
            openDocumentLauncher.launch(arrayOf("application/zip"))
        }
    }

    private fun updateUiStates() {
        val isNormalInstallChecked = switchNormalInstall.isChecked
        val isUseRootInstallChecked = switchUseRootInstall.isChecked

        textViewRootSettingsLabel.isEnabled = !isNormalInstallChecked
        switchUseRootInstall.isEnabled = !isNormalInstallChecked

        val rootOptionsActive = !isNormalInstallChecked && isUseRootInstallChecked

        textViewRootInstallMethodLabel.isEnabled = rootOptionsActive
        radioPmInstall.isEnabled = rootOptionsActive
        radioCopyToSystem.isEnabled = rootOptionsActive
        for (i in 0 until radioGroupRootInstallMethod.childCount) {
            (radioGroupRootInstallMethod.getChildAt(i) as? RadioButton)?.isEnabled = rootOptionsActive
        }

        val copyToSystemMethodActive = rootOptionsActive && radioCopyToSystem.isChecked
        textViewCopyToSystemLocationLabel.isEnabled = copyToSystemMethodActive
        radioSystemApp.isEnabled = copyToSystemMethodActive
        radioSystemPrivApp.isEnabled = copyToSystemMethodActive
        for (i in 0 until radioGroupInstallLocation.childCount) {
            (radioGroupInstallLocation.getChildAt(i) as? RadioButton)?.isEnabled = copyToSystemMethodActive
        }
    }

    private fun loadPreferences() {
        switchNormalInstall.isChecked = sharedPreferences.getBoolean(KEY_NORMAL_INSTALL_MODE, false)
        val normalInstallIsCurrentlyChecked = switchNormalInstall.isChecked
        switchUseRootInstall.isChecked = if (normalInstallIsCurrentlyChecked) false else sharedPreferences.getBoolean(KEY_USE_ROOT_INSTALL, true)

        val rootMethod = sharedPreferences.getString(KEY_ROOT_INSTALL_METHOD, ROOT_METHOD_PM_INSTALL)
        if (rootMethod == ROOT_METHOD_COPY_TO_SYSTEM) radioCopyToSystem.isChecked = true else radioPmInstall.isChecked = true

        val location = sharedPreferences.getString(KEY_INSTALL_LOCATION, "system/app")
        if ("system/priv-app" == location) radioSystemPrivApp.isChecked = true else radioSystemApp.isChecked = true

        switchEnableSendFeedback.isChecked = sharedPreferences.getBoolean(KEY_ENABLE_SEND_FEEDBACK, true)
        editTextAboutText.setText(sharedPreferences.getString(KEY_ABOUT_TEXT, DEFAULT_ABOUT_TEXT))
    }

    private fun savePreferences() {
        sharedPreferences.edit {
            putBoolean(KEY_NORMAL_INSTALL_MODE, switchNormalInstall.isChecked)
            putBoolean(KEY_USE_ROOT_INSTALL, switchUseRootInstall.isChecked)
            val rootMethod = if (radioCopyToSystem.isChecked) ROOT_METHOD_COPY_TO_SYSTEM else ROOT_METHOD_PM_INSTALL
            putString(KEY_ROOT_INSTALL_METHOD, rootMethod)
            val installLocation = if (radioSystemPrivApp.isChecked) "system/priv-app" else "system/app"
            putString(KEY_INSTALL_LOCATION, installLocation)
            putBoolean(KEY_ENABLE_SEND_FEEDBACK, switchEnableSendFeedback.isChecked)
            putString(KEY_ABOUT_TEXT, editTextAboutText.text.toString())
        }
    }

    private fun showAddAppDialog() {
        val builder = AlertDialog.Builder(this)
        val inflater = this.layoutInflater
        val dialogView = inflater.inflate(R.layout.dialog_add_app, null)
        builder.setView(dialogView)

        editTextDialogAppName = dialogView.findViewById(R.id.editTextNewAppName)
        editTextDialogAppDescription = dialogView.findViewById(R.id.editTextNewAppDescription)
        editTextDialogAppPackageName = dialogView.findViewById(R.id.editTextNewAppPackageName)
        val buttonSelectApkFromDialog: Button = dialogView.findViewById(R.id.buttonSelectApk)
        val buttonSelectImageFromDialog: Button = dialogView.findViewById(R.id.buttonSelectImage)

        currentApkUri = null
        currentImageUri = null

        buttonSelectApkFromDialog.setOnClickListener {
            pickApkLauncher.launch(arrayOf("application/vnd.android.package-archive"))
        }
        buttonSelectImageFromDialog.setOnClickListener {
            pickImageLauncher.launch(arrayOf("image/*"))
        }

        builder.setTitle("הוסף אפליקציה חדשה")
        builder.setPositiveButton("הוסף") { dialog, _ ->
            val appName = editTextDialogAppName?.text.toString().trim()
            val appDescription = editTextDialogAppDescription?.text.toString().trim()
            val appPackageNameText = editTextDialogAppPackageName?.text.toString().trim()

            if (TextUtils.isEmpty(appName) || TextUtils.isEmpty(appDescription) || TextUtils.isEmpty(appPackageNameText) || currentApkUri == null) {
                Toast.makeText(this, "אנא מלא את כל השדות, בחר קובץ APK, והזן שם חבילה.", Toast.LENGTH_LONG).show()
                return@setPositiveButton
            }

            val originalApkFileName = getFileNameFromUri(currentApkUri!!)
            val savedApkFileName = saveFileToInternalStorage(currentApkUri!!, originalApkFileName)
            var savedImageFileName: String? = null
            currentImageUri?.let { uri ->
                val originalImageFileName = getFileNameFromUri(uri)
                savedImageFileName = saveFileToInternalStorage(uri, originalImageFileName)
            }

            if (savedApkFileName != null) {
                val newApp = AppInfo(appName, appDescription, savedApkFileName, savedImageFileName, appPackageNameText)
                val currentStoredApps = loadAppsListFromPrefs().toMutableList()
                currentStoredApps.add(newApp)
                saveAppsListDirectlyToPrefs(currentStoredApps)
                Toast.makeText(this, "אפליקציה '$appName' נוספה", Toast.LENGTH_SHORT).show()
                setResult(Activity.RESULT_OK)
            } else {
                Toast.makeText(this, "שגיאה קריטית בשמירת קובץ APK", Toast.LENGTH_LONG).show()
            }
            dialog.dismiss()
        }
        builder.setNegativeButton("ביטול") { dialog, _ -> dialog.cancel() }
        builder.show()
    }

    private fun saveAppsListDirectlyToPrefs(updatedAppList: List<AppInfo>) {
        val jsonApps = gson.toJson(updatedAppList)
        sharedPreferences.edit { putString(KEY_APPS_LIST, jsonApps) }
    }

    private fun loadAppsListFromPrefs(): List<AppInfo> {
        val jsonApps = sharedPreferences.getString(KEY_APPS_LIST, null)
        return if (jsonApps != null) {
            val typeToken: Type = object : TypeToken<ArrayList<AppInfo>>() {}.type
            try { gson.fromJson(jsonApps, typeToken) ?: ArrayList() }
            catch (e: Exception) { Log.e(TAG, "Error parsing app list from prefs", e); ArrayList() }
        } else ArrayList()
    }

    private fun getFileNameFromUri(uri: Uri): String {
        var fileName: String? = null
        if (uri.scheme == "content") {
            try {
                contentResolver.query(uri, null, null, null, null)?.use { cursor ->
                    if (cursor.moveToFirst()) {
                        fileName = cursor.getString(cursor.getColumnIndexOrThrow(OpenableColumns.DISPLAY_NAME))
                    }
                }
            } catch (e: Exception) { Log.e(TAG, "Error getting file name from content URI", e) }
        }
        if (fileName == null) {
            fileName = uri.path?.substringAfterLast('/')
        }
        return fileName ?: "unknown_file_${UUID.randomUUID()}.tmp"
    }

    private fun saveFileToInternalStorage(uri: Uri, desiredFileName: String): String? {
        try {
            val inputStream: InputStream? = contentResolver.openInputStream(uri)
            if (inputStream == null) { Log.e(TAG, "InputStream is null for URI: $uri"); return null }
            val file = File(filesDir, desiredFileName)
            FileOutputStream(file).use { output -> inputStream.use { input -> input.copyTo(output) } }
            Log.d(TAG, "File saved to: ${file.absolutePath}")
            return file.name
        } catch (e: Exception) {
            Log.e(TAG, "Error saving file: $desiredFileName from URI: $uri", e)
            Toast.makeText(this, "שגיאה בשמירת קובץ: ${e.message}", Toast.LENGTH_LONG).show()
            return null
        }
    }

    private fun changePassword() {
        // ... (יישום מלא כמו קודם)
    }


    // --- Export/Import Logic ---
    private fun performExportStore(uri: Uri) {
        Toast.makeText(this, "מתחיל ייצוא...", Toast.LENGTH_SHORT).show()
        try {
            contentResolver.openOutputStream(uri)?.use { outputStream ->
                ZipOutputStream(BufferedOutputStream(outputStream)).use { zos ->
                    val appListToExport = loadAppsListFromPrefs()
                    val config = StoreConfig(
                        appList = appListToExport,
                        settingsPassword = sharedPreferences.getString(KEY_SETTINGS_PASSWORD, DEFAULT_PASSWORD),
                        normalInstallMode = sharedPreferences.getBoolean(KEY_NORMAL_INSTALL_MODE, false),
                        useRootInstall = sharedPreferences.getBoolean(KEY_USE_ROOT_INSTALL, true),
                        rootInstallMethod = sharedPreferences.getString(KEY_ROOT_INSTALL_METHOD, ROOT_METHOD_PM_INSTALL),
                        installLocation = sharedPreferences.getString(KEY_INSTALL_LOCATION, "system/app"),
                        enableSendFeedback = sharedPreferences.getBoolean(KEY_ENABLE_SEND_FEEDBACK, true),
                        aboutText = sharedPreferences.getString(KEY_ABOUT_TEXT, DEFAULT_ABOUT_TEXT)
                    )
                    val manifestJson = gson.toJson(config)

                    zos.putNextEntry(ZipEntry(MANIFEST_FILE_NAME))
                    zos.write(manifestJson.toByteArray())
                    zos.closeEntry()

                    appListToExport.forEach { appInfo ->
                        if (appInfo.apkFileName.isNotEmpty()) {
                            val apkFile = File(filesDir, appInfo.apkFileName)
                            if (apkFile.exists()) addFileToZip(apkFile, "$APKS_DIR_NAME/${appInfo.apkFileName}", zos)
                        }
                        appInfo.imageFileName?.let { imgName ->
                            if (imgName.isNotEmpty()) {
                                val imageFile = File(filesDir, imgName)
                                if (imageFile.exists()) addFileToZip(imageFile, "$IMAGES_DIR_NAME/$imgName", zos)
                            }
                        }
                    }
                }
            }
            Toast.makeText(this, "הייצוא הושלם בהצלחה!", Toast.LENGTH_LONG).show()
        } catch (e: Exception) {
            Log.e(TAG, "Error during export", e)
            Toast.makeText(this, "שגיאה במהלך הייצוא: ${e.message}", Toast.LENGTH_LONG).show()
        }
    }

    private fun addFileToZip(file: File, entryName: String, zos: ZipOutputStream) {
        FileInputStream(file).use { fis ->
            BufferedInputStream(fis).use { bis ->
                val entry = ZipEntry(entryName)
                zos.putNextEntry(entry)
                bis.copyTo(zos)
                zos.closeEntry()
                Log.d(TAG, "Added to ZIP: $entryName")
            }
        }
    }

    private fun performImportStore(uri: Uri) {
        Toast.makeText(this, "מתחיל ייבוא...", Toast.LENGTH_SHORT).show()
        var manifestConfig: StoreConfig? = null
        val importedApkFilesMap = mutableMapOf<String, File>()
        val importedImageFilesMap = mutableMapOf<String, File>()
        val tempDir = File(cacheDir, "import_temp_${System.currentTimeMillis()}")
        if (!tempDir.exists()) tempDir.mkdirs()

        try {
            contentResolver.openInputStream(uri)?.use { inputStream ->
                ZipInputStream(BufferedInputStream(inputStream)).use { zis ->
                    var entry: ZipEntry? = zis.nextEntry
                    while (entry != null) {
                        val entryName = entry.name
                        if (!entry.isDirectory) {
                            val tempFile = File(tempDir, entryName.substringAfterLast('/'))
                            FileOutputStream(tempFile).use { fos -> zis.copyTo(fos) }
                            Log.d(TAG, "Extracted to temp: ${tempFile.absolutePath} from ZIP entry: $entryName")
                            when {
                                entryName == MANIFEST_FILE_NAME -> {
                                    val manifestJson = tempFile.readText()
                                    manifestConfig = gson.fromJson(manifestJson, StoreConfig::class.java)
                                    tempFile.delete()
                                }
                                entryName.startsWith("$APKS_DIR_NAME/") -> importedApkFilesMap[entryName.substringAfter("$APKS_DIR_NAME/")] = tempFile
                                entryName.startsWith("$IMAGES_DIR_NAME/") -> importedImageFilesMap[entryName.substringAfter("$IMAGES_DIR_NAME/")] = tempFile
                                else -> tempFile.delete()
                            }
                        }
                        zis.closeEntry()
                        entry = zis.nextEntry
                    }
                }
            }

            val currentConfig = manifestConfig ?: run {
                Toast.makeText(this, "שגיאה: קובץ מניפסט לא נמצא או פגום ב-ZIP.", Toast.LENGTH_LONG).show()
                return
            }

            val oldApps = loadAppsListFromPrefs()
            oldApps.forEach { oldApp ->
                File(filesDir, oldApp.apkFileName).delete()
                oldApp.imageFileName?.let { File(filesDir, it).delete() }
            }

            val newAppList = currentConfig.appList.mapNotNull { appInfoFromManifest ->
                var finalApkPathInApp: String? = null
                var finalImagePathInApp: String? = null

                importedApkFilesMap[appInfoFromManifest.apkFileName]?.let { tempApkFile ->
                    val destApkFile = File(filesDir, appInfoFromManifest.apkFileName)
                    if(tempApkFile.renameTo(destApkFile)) { finalApkPathInApp = destApkFile.name }
                    else { Log.e(TAG, "Failed to move APK: ${tempApkFile.name}") }
                }
                appInfoFromManifest.imageFileName?.let { manifestImageName ->
                    importedImageFilesMap[manifestImageName]?.let { tempImageFile ->
                        val destImageFile = File(filesDir, manifestImageName)
                        if(tempImageFile.renameTo(destImageFile)) { finalImagePathInApp = destImageFile.name }
                        else { Log.e(TAG, "Failed to move Image: ${tempImageFile.name}") }
                    }
                }
                if (finalApkPathInApp != null) {
                    appInfoFromManifest.copy(apkFileName = finalApkPathInApp!!, imageFileName = finalImagePathInApp)
                } else {
                    Log.w(TAG, "Skipping app ${appInfoFromManifest.name} due to missing APK file after import.")
                    null
                }
            }

            saveAppsListDirectlyToPrefs(newAppList)

            sharedPreferences.edit {
                putString(KEY_SETTINGS_PASSWORD, currentConfig.settingsPassword ?: DEFAULT_PASSWORD)
                putBoolean(KEY_NORMAL_INSTALL_MODE, currentConfig.normalInstallMode)
                putBoolean(KEY_USE_ROOT_INSTALL, currentConfig.useRootInstall)
                putString(KEY_ROOT_INSTALL_METHOD, currentConfig.rootInstallMethod ?: ROOT_METHOD_PM_INSTALL)
                putString(KEY_INSTALL_LOCATION, currentConfig.installLocation ?: "system/app")
                putBoolean(KEY_ENABLE_SEND_FEEDBACK, currentConfig.enableSendFeedback)
                putString(KEY_ABOUT_TEXT, currentConfig.aboutText ?: DEFAULT_ABOUT_TEXT)
            }

            Toast.makeText(this, "הייבוא הושלם בהצלחה!", Toast.LENGTH_LONG).show()
            loadPreferences()
            updateUiStates()
            setResult(Activity.RESULT_OK)
        } catch (e: Exception) {
            Log.e(TAG, "Error during import", e)
            Toast.makeText(this, "שגיאה במהלך הייבוא: ${e.message}", Toast.LENGTH_LONG).show()
        } finally {
            tempDir.deleteRecursively()
        }
    }
}